# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation.  All rights reserved.
# Licensed under the MIT License.  See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import math

import numpy as np
import onnx
from onnx import TensorProto, helper
from packaging import version


def float_tensor(name: str, shape: list[int], random=False):
    low = 0.0
    high = 1.0
    total_elements = 1
    for x in shape:
        total_elements *= x
    weights = [np.random.uniform(low, high) for _ in range(total_elements)] if random else [1.0] * total_elements
    return helper.make_tensor(name, TensorProto.FLOAT, shape, weights)


def reverse_if(inputs, reverse=False):
    if reverse:
        inputs.reverse()
    return inputs


def create_bert_attention(
    input_hidden_size=16,
    num_heads=2,
    pruned_qk_hidden_size=16,
    pruned_v_hidden_size=16,
    use_float_mask=False,
    switch_add_inputs=False,
):
    # unsqueeze in opset version 13 has two inputs (axis is moved from attribute to input).
    has_unsqueeze_two_inputs = version.parse(onnx.__version__) >= version.parse("1.8.0")

    # nodes in attention subgraph
    nodes = [
        helper.make_node("Add", ["input_1", "input_2"], ["layernorm_input"], "add_layernorm"),
        helper.make_node(
            "LayerNormalization",
            ["layernorm_input", "layer_norm_weight", "layer_norm_bias"],
            ["layernorm_out"],
            "layernorm",
            axis=-1,
            epsion=0.000009999999747378752,
        ),
        # q nodes
        helper.make_node("MatMul", ["layernorm_out", "matmul_q_weight"], ["matmul_q_out"], "matmul_q"),
        helper.make_node(
            "Add",
            reverse_if(["matmul_q_out", "add_q_weight"], switch_add_inputs),
            ["add_q_out"],
            "add_q",
        ),
        helper.make_node(
            "Reshape",
            ["add_q_out", "reshape_weight_qk"],
            ["reshape_q_out"],
            "reshape_q",
        ),
        helper.make_node(
            "Transpose",
            ["reshape_q_out"],
            ["transpose_q_out"],
            "transpose_q",
            perm=[0, 2, 1, 3],
        ),
        # k nodes
        helper.make_node("MatMul", ["layernorm_out", "matmul_k_weight"], ["matmul_k_out"], "matmul_k"),
        helper.make_node(
            "Add",
            reverse_if(["matmul_k_out", "add_k_weight"], switch_add_inputs),
            ["add_k_out"],
            "add_k",
        ),
        helper.make_node(
            "Reshape",
            ["add_k_out", "reshape_weight_qk"],
            ["reshape_k_out"],
            "reshape_k",
        ),
        helper.make_node(
            "Transpose",
            ["reshape_k_out"],
            ["transpose_k_out"],
            "transpose_k",
            perm=[0, 2, 3, 1],
        ),
        # mask nodes
        (
            helper.make_node("Unsqueeze", ["input_mask", "axes_1"], ["unsqueeze0_out"], "unsqueeze0")
            if has_unsqueeze_two_inputs
            else helper.make_node("Unsqueeze", ["input_mask"], ["unsqueeze0_out"], "unsqueeze0", axes=[1])
        ),
        (
            helper.make_node("Unsqueeze", ["unsqueeze0_out", "axes_2"], ["unsqueeze1_out"], "unsqueeze1")
            if has_unsqueeze_two_inputs
            else helper.make_node("Unsqueeze", ["unsqueeze0_out"], ["unsqueeze1_out"], "unsqueeze1", axes=[2])
        ),
        # when attention_mask is float type, no need to cast
        helper.make_node("Cast", ["unsqueeze1_out"], ["cast_out"], "cast", to=1) if not use_float_mask else None,
        helper.make_node(
            "Sub",
            ["sub_weight", "unsqueeze1_out" if use_float_mask else "cast_out"],
            ["sub_out"],
            "sub",
        ),
        helper.make_node("Mul", ["sub_out", "mul_weight"], ["mul_mask_out"], "mul_mask"),
        # qk nodes
        helper.make_node(
            "MatMul",
            ["transpose_q_out", "transpose_k_out"],
            ["matmul_qk_out"],
            "matmul_qk",
        ),
        helper.make_node("Div", ["matmul_qk_out", "div_weight"], ["div_qk_out"], "div_qk"),
        helper.make_node(
            "Add",
            reverse_if(["div_qk_out", "mul_mask_out"], switch_add_inputs),
            ["add_qk_out"],
            "add_qk",
        ),
        helper.make_node("Softmax", ["add_qk_out"], ["softmax_qk_out"], "softmax_qk", axis=3),
        # v nodes
        helper.make_node("MatMul", ["layernorm_out", "matmul_v_weight"], ["matmul_v_out"], "matmul_v"),
        helper.make_node("Add", ["matmul_v_out", "add_v_weight"], ["add_v_out"], "add_v"),
        helper.make_node("Reshape", ["add_v_out", "reshape_weight_v"], ["reshape_v_out"], "reshape_v"),
        helper.make_node(
            "Transpose",
            ["reshape_v_out"],
            ["transpose_v_out"],
            "transpose_v",
            perm=[0, 2, 1, 3],
        ),
        # qkv nodes
        helper.make_node(
            "MatMul",
            ["softmax_qk_out", "transpose_v_out"],
            ["matmul_qkv_1_out"],
            "matmul_qkv_1",
        ),
        helper.make_node(
            "Transpose",
            ["matmul_qkv_1_out"],
            ["transpose_qkv_out"],
            "transpose_qkv",
            perm=[0, 2, 1, 3],
        ),
        helper.make_node(
            "Reshape",
            ["transpose_qkv_out", "reshape_weight_qkv"],
            ["reshape_qkv_out"],
            "reshape_qkv",
        ),
        helper.make_node(
            "MatMul",
            ["reshape_qkv_out", "matmul_qkv_weight"],
            ["matmul_qkv_2_out"],
            "matmul_qkv_2",
        ),
        helper.make_node(
            "Add",
            reverse_if(["matmul_qkv_2_out", "add_qkv_weight"], switch_add_inputs),
            ["add_qkv_out"],
            "add_qkv",
        ),
        helper.make_node(
            "Add",
            reverse_if(["add_qkv_out", "layernorm_out"], switch_add_inputs),
            ["skip_output"],
            "add_skip",
        ),
        helper.make_node(
            "LayerNormalization",
            ["skip_output", "layer_norm_weight", "layer_norm_bias"],
            ["output"],
            "layernorm2",
            axis=-1,
            epsion=0.000009999999747378752,
        ),
    ]

    pruned_qk_head_size = int(pruned_qk_hidden_size / num_heads)
    pruned_v_head_size = int(pruned_v_hidden_size / num_heads)
    initializers = [  # initializers
        float_tensor("layer_norm_weight", [input_hidden_size]),
        float_tensor("layer_norm_bias", [input_hidden_size]),
        float_tensor("matmul_q_weight", [input_hidden_size, pruned_qk_hidden_size]),
        float_tensor("matmul_k_weight", [input_hidden_size, pruned_qk_hidden_size]),
        float_tensor("matmul_v_weight", [input_hidden_size, pruned_v_hidden_size]),
        float_tensor("matmul_qkv_weight", [pruned_v_hidden_size, input_hidden_size]),
        float_tensor("add_q_weight", [pruned_qk_hidden_size]),
        float_tensor("add_k_weight", [pruned_qk_hidden_size]),
        float_tensor("add_v_weight", [pruned_v_hidden_size]),
        float_tensor("add_qkv_weight", [input_hidden_size]),
        helper.make_tensor("div_weight", TensorProto.FLOAT, [1], [math.sqrt(pruned_qk_head_size)]),
        helper.make_tensor("sub_weight", TensorProto.FLOAT, [1], [1.0]),
        helper.make_tensor("mul_weight", TensorProto.FLOAT, [1], [-10000]),
        helper.make_tensor(
            "reshape_weight_qk",
            TensorProto.INT64,
            [4],
            [0, 0, num_heads, pruned_qk_head_size],
        ),
        helper.make_tensor(
            "reshape_weight_v",
            TensorProto.INT64,
            [4],
            [0, 0, num_heads, pruned_v_head_size],
        ),
        helper.make_tensor("reshape_weight_qkv", TensorProto.INT64, [3], [0, 0, pruned_v_hidden_size]),
    ]

    if has_unsqueeze_two_inputs:
        initializers.append(helper.make_tensor("axes_1", TensorProto.INT64, [1], [1]))
        initializers.append(helper.make_tensor("axes_2", TensorProto.INT64, [1], [2]))

    batch_size = 1
    sequence_length = 3
    graph = helper.make_graph(
        [node for node in nodes if node],
        "AttentionFusionPrunedModel",  # name
        [  # inputs
            helper.make_tensor_value_info(
                "input_1",
                TensorProto.FLOAT,
                [batch_size, sequence_length, input_hidden_size],
            ),
            helper.make_tensor_value_info(
                "input_2",
                TensorProto.FLOAT,
                [batch_size, sequence_length, input_hidden_size],
            ),
            helper.make_tensor_value_info(
                "input_mask",
                TensorProto.FLOAT if use_float_mask else TensorProto.INT64,
                [batch_size, sequence_length],
            ),
        ],
        [  # outputs
            helper.make_tensor_value_info(
                "output",
                TensorProto.FLOAT,
                [batch_size, sequence_length, input_hidden_size],
            ),
        ],
        initializers,
    )

    # Needed so that we don't see the new LayerNormalization function added in version 17.
    # TODO(https://github.com/microsoft/onnxruntime/issues/11916): Remove once fixed.
    opsetid = helper.make_opsetid("ai.onnx", min(onnx.defs.onnx_opset_version(), 16))
    return helper.make_model(graph, opset_imports=(opsetid,))


def create_tf2onnx_attention_3d(input_hidden_size=16, num_heads=4, head_size=4, use_float_mask=False):
    # unsqueeze in opset version 13 has two inputs (axis is moved from attribute to input).
    has_unsqueeze_two_inputs = version.parse(onnx.__version__) >= version.parse("1.8.0")

    # nodes in attention subgraph
    nodes = [
        helper.make_node("Add", ["input_1", "input_2"], ["layernorm_input"], "add_layernorm"),
        helper.make_node(
            "LayerNormalization",
            ["layernorm_input", "layer_norm_weight", "layer_norm_bias"],
            ["layernorm_out"],
            "layernorm",
            axis=-1,
            epsion=0.000009999999747378752,
        ),
        # q nodes
        helper.make_node(
            "Einsum",
            ["layernorm_out", "einsum_q_weight"],
            ["einsum_q_out"],
            "einsum_q",
            equation="abc,cde->abde",
        ),
        helper.make_node("Add", ["einsum_q_out", "add_q_weight"], ["add_q_out"], "add_q"),
        # k nodes
        helper.make_node(
            "Einsum",
            ["layernorm_out", "einsum_k_weight"],
            ["einsum_k_out"],
            "einsum_k",
            equation="abc,cde->abde",
        ),
        helper.make_node("Add", ["einsum_k_out", "add_k_weight"], ["add_k_out"], "add_k"),
        helper.make_node("Mul", ["add_k_out", "mul_weight_1"], ["mul_k_out"], "mul_k"),
        # mask nodes
        (
            helper.make_node("Unsqueeze", ["input_mask", "axes_1"], ["unsqueeze0_out"], "unsqueeze0")
            if has_unsqueeze_two_inputs
            else helper.make_node("Unsqueeze", ["input_mask"], ["unsqueeze0_out"], "unsqueeze0", axes=[1, 2])
        ),
        helper.make_node(
            "Slice",
            ["unsqueeze0_out", "slice_start", "slice_end", "slice_axes", "slice_steps"],
            ["slice_out"],
            "slice",
        ),
        # when attention_mask is float type, no need to cast
        helper.make_node("Cast", ["slice_out"], ["cast_out"], "cast", to=1) if not use_float_mask else None,
        helper.make_node(
            "Sub",
            ["sub_weight", "unsqueeze1_out" if use_float_mask else "cast_out"],
            ["sub_out"],
            "sub",
        ),
        helper.make_node("Mul", ["sub_out", "mul_weight_2"], ["mul_mask_out"], "mul_mask"),
        # qk nodes
        helper.make_node(
            "Einsum",
            ["add_q_out", "mul_k_out"],
            ["einsum_qk_out"],
            "einsum_qk",
            equation="aecd,abcd->acbe",
        ),
        helper.make_node("Add", ["einsum_qk_out", "mul_mask_out"], ["add_qk_out"], "add_qk"),
        helper.make_node("Softmax", ["add_qk_out"], ["softmax_qk_out"], "softmax_qk", axis=3),
        # v nodes
        helper.make_node(
            "Einsum",
            ["layernorm_out", "einsum_v_weight"],
            ["einsum_v_out"],
            "einsum_v",
            equation="abc,cde->abde",
        ),
        helper.make_node("Add", ["einsum_v_out", "add_v_weight"], ["add_v_out"], "add_v"),
        # qkv nodes
        helper.make_node(
            "Einsum",
            ["softmax_qk_out", "add_v_out"],
            ["einsum_qkv_1_out"],
            "einsum_qkv_1",
            equation="acbe,aecd->abcd",
        ),
        helper.make_node(
            "Einsum",
            ["einsum_qkv_1_out", "einsum_qkv_weight"],
            ["einsum_qkv_2_out"],
            "einsum_qkv_2",
            equation="abcd,cde->abe",
        ),
        helper.make_node("Add", ["einsum_qkv_2_out", "add_qkv_weight"], ["add_qkv_out"], "add_qkv"),
        helper.make_node("Add", ["add_qkv_out", "layernorm_out"], ["skip_output"], "add_skip"),
        helper.make_node(
            "LayerNormalization",
            ["skip_output", "layer_norm_weight", "layer_norm_bias"],
            ["output"],
            "layernorm2",
            axis=-1,
            epsion=0.000009999999747378752,
        ),
    ]

    initializers = [  # initializers
        float_tensor("layer_norm_weight", [input_hidden_size]),
        float_tensor("layer_norm_bias", [input_hidden_size]),
        float_tensor("einsum_q_weight", [input_hidden_size, num_heads, head_size]),
        float_tensor("einsum_k_weight", [input_hidden_size, num_heads, head_size]),
        float_tensor("einsum_v_weight", [input_hidden_size, num_heads, head_size]),
        float_tensor("einsum_qkv_weight", [num_heads, head_size, input_hidden_size]),
        float_tensor("add_q_weight", [num_heads, head_size]),
        float_tensor("add_k_weight", [num_heads, head_size]),
        float_tensor("add_v_weight", [num_heads, head_size]),
        float_tensor("add_qkv_weight", [input_hidden_size]),
        helper.make_tensor("sub_weight", TensorProto.FLOAT, [1], [1.0]),
        helper.make_tensor("mul_weight_1", TensorProto.FLOAT, [1], [-10000]),
        helper.make_tensor("mul_weight_2", TensorProto.FLOAT, [1], [0.125]),
        helper.make_tensor("reshape_weight_1", TensorProto.INT64, [4], [0, 0, num_heads, head_size]),
        helper.make_tensor("slice_start", TensorProto.INT32, [4], [0, 0, 0, 0]),
        helper.make_tensor(
            "slice_end",
            TensorProto.INT32,
            [4],
            [1000000000, 1000000000, 1000000000, 1000000000],
        ),
        helper.make_tensor("slice_axes", TensorProto.INT32, [4], [0, 1, 2, 3]),
        helper.make_tensor("slice_steps", TensorProto.INT32, [4], [1, 1, 1, 1]),
    ]

    if has_unsqueeze_two_inputs:
        initializers.append(helper.make_tensor("axes_1", TensorProto.INT64, [2], [1, 2]))

    batch_size = 1
    sequence_length = 3
    graph = helper.make_graph(
        [node for node in nodes if node],
        "AttentionFusionPrunedModel",  # name
        [  # inputs
            helper.make_tensor_value_info(
                "input_1",
                TensorProto.FLOAT,
                [batch_size, sequence_length, input_hidden_size],
            ),
            helper.make_tensor_value_info(
                "input_2",
                TensorProto.FLOAT,
                [batch_size, sequence_length, input_hidden_size],
            ),
            helper.make_tensor_value_info(
                "input_mask",
                TensorProto.FLOAT if use_float_mask else TensorProto.INT64,
                [batch_size, sequence_length],
            ),
        ],
        [  # outputs
            helper.make_tensor_value_info(
                "output",
                TensorProto.FLOAT,
                [batch_size, sequence_length, input_hidden_size],
            ),
        ],
        initializers,
    )

    return helper.make_model(graph)


if __name__ == "__main__":
    model = create_bert_attention()
    onnx.save(model, "pruned_bert_attention.onnx")
    model = create_bert_attention(switch_add_inputs=True)
    onnx.save(model, "bert_attention_reverse_add_order.onnx")
    model = create_tf2onnx_attention_3d()
    onnx.save(model, "bert_3d_attention.onnx")



import argparse
import os

import onnx


def export_and_recurse(node, attribute, output_dir, level):
    name = node.name
    name = name.replace("/", "_")
    sub_model = onnx.ModelProto()
    sub_model.graph.MergeFrom(attribute.g)
    filename = "L" + str(level) + "_" + node.op_type + "_" + attribute.name + "_" + name + ".onnx"
    onnx.save_model(sub_model, os.path.join(output_dir, filename))
    dump_subgraph(sub_model, output_dir, level + 1)


def dump_subgraph(model, output_dir, level=0):
    graph = model.graph

    for node in graph.node:
        if node.op_type == "Scan" or node.op_type == "Loop":
            body_attribute = next(iter(filter(lambda attr: attr.name == "body", node.attribute)))
            export_and_recurse(node, body_attribute, output_dir, level)
        if node.op_type == "If":
            then_attribute = next(iter(filter(lambda attr: attr.name == "then_branch", node.attribute)))
            else_attribute = next(iter(filter(lambda attr: attr.name == "else_branch", node.attribute)))
            export_and_recurse(node, then_attribute, output_dir, level)
            export_and_recurse(node, else_attribute, output_dir, level)


def parse_args():
    parser = argparse.ArgumentParser(
        os.path.basename(__file__), description="Dump all subgraphs from an ONNX model into separate onnx files."
    )
    parser.add_argument("-m", "--model", required=True, help="model file")
    parser.add_argument("-o", "--out", required=True, help="output directory")
    return parser.parse_args()


def main():
    args = parse_args()

    model_path = args.model
    out = os.path.abspath(args.out)

    if not os.path.exists(out):
        os.makedirs(out)

    model = onnx.load_model(model_path)
    dump_subgraph(model, out)


if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

"""

.. _l-example-backend-api:

ONNX Runtime Backend for ONNX
=============================

*ONNX Runtime* extends the
`onnx backend API <https://github.com/onnx/onnx/blob/main/docs/ImplementingAnOnnxBackend.md>`_
to run predictions using this runtime.
Let's use the API to compute the prediction
of a simple logistic regression model.
"""

import numpy as np
from onnx import load

import onnxruntime.backend as backend

########################################
# The device depends on how the package was compiled,
# GPU or CPU.
from onnxruntime import datasets, get_device
from onnxruntime.capi.onnxruntime_pybind11_state import InvalidArgument

device = get_device()

name = datasets.get_example("logreg_iris.onnx")
model = load(name)

rep = backend.prepare(model, device)
x = np.array([[-1.0, -2.0]], dtype=np.float32)
try:
    label, proba = rep.run(x)
    print(f"label={label}")
    print(f"probabilities={proba}")
except (RuntimeError, InvalidArgument) as e:
    print(e)

########################################
# The backend can also directly load the model
# without using *onnx*.

rep = backend.prepare(name, device)
x = np.array([[-1.0, -2.0]], dtype=np.float32)
try:
    label, proba = rep.run(x)
    print(f"label={label}")
    print(f"probabilities={proba}")
except (RuntimeError, InvalidArgument) as e:
    print(e)

#######################################
# The backend API is implemented by other frameworks
# and makes it easier to switch between multiple runtimes
# with the same API.



import copy
import logging
from collections import OrderedDict
from collections.abc import Mapping
from typing import Any

import numpy
import torch

from onnxruntime import InferenceSession, RunOptions

# Type alias
ShapeDict = Mapping[str, tuple | list[int]]

logger = logging.getLogger(__name__)


class TypeHelper:
    @staticmethod
    def get_input_type(ort_session: InferenceSession, name: str) -> str:
        for _i, input in enumerate(ort_session.get_inputs()):
            if input.name == name:
                return input.type
        raise ValueError(f"input name {name} not found")

    @staticmethod
    def get_output_type(ort_session, name: str) -> str:
        for _i, output in enumerate(ort_session.get_outputs()):
            if output.name == name:
                return output.type

        raise ValueError(f"output name {name} not found")

    @staticmethod
    def ort_type_to_numpy_type(ort_type: str):
        ort_type_to_numpy_type_map = {
            "tensor(int64)": numpy.longlong,
            "tensor(int32)": numpy.intc,
            "tensor(float)": numpy.float32,
            "tensor(float16)": numpy.float16,
            "tensor(bool)": bool,
        }
        if ort_type not in ort_type_to_numpy_type_map:
            raise ValueError(f"{ort_type} not found in map")

        return ort_type_to_numpy_type_map[ort_type]

    @staticmethod
    def ort_type_to_torch_type(ort_type: str):
        ort_type_to_torch_type_map = {
            "tensor(int64)": torch.int64,
            "tensor(int32)": torch.int32,
            "tensor(float)": torch.float32,
            "tensor(float16)": torch.float16,
            "tensor(bool)": torch.bool,
        }
        if ort_type not in ort_type_to_torch_type_map:
            raise ValueError(f"{ort_type} not found in map")

        return ort_type_to_torch_type_map[ort_type]

    @staticmethod
    def numpy_type_to_torch_type(numpy_type: numpy.dtype):
        numpy_type_to_torch_type_map = {
            numpy.longlong: torch.int64,
            numpy.intc: torch.int32,
            numpy.int32: torch.int32,
            numpy.float32: torch.float32,
            numpy.float16: torch.float16,
            bool: torch.bool,
        }
        if numpy_type not in numpy_type_to_torch_type_map:
            raise ValueError(f"{numpy_type} not found in map")

        return numpy_type_to_torch_type_map[numpy_type]

    @staticmethod
    def torch_type_to_numpy_type(torch_type: torch.dtype):
        torch_type_to_numpy_type_map = {
            torch.int64: numpy.longlong,
            torch.int32: numpy.intc,
            torch.float32: numpy.float32,
            torch.float16: numpy.float16,
            torch.bool: bool,
        }
        if torch_type not in torch_type_to_numpy_type_map:
            raise ValueError(f"{torch_type} not found in map")

        return torch_type_to_numpy_type_map[torch_type]

    @staticmethod
    def get_io_numpy_type_map(ort_session: InferenceSession) -> dict[str, numpy.dtype]:
        """Create a mapping from input/output name to numpy data type"""
        name_to_numpy_type = {}
        for input in ort_session.get_inputs():
            name_to_numpy_type[input.name] = TypeHelper.ort_type_to_numpy_type(input.type)

        for output in ort_session.get_outputs():
            name_to_numpy_type[output.name] = TypeHelper.ort_type_to_numpy_type(output.type)
        return name_to_numpy_type


class IOBindingHelper:
    @staticmethod
    def get_output_buffers(ort_session: InferenceSession, output_shapes, device):
        """Returns a dictionary of output name as key, and 1D tensor as value. The tensor has enough space for given shape."""
        output_buffers = {}
        for name, shape in output_shapes.items():
            ort_type = TypeHelper.get_output_type(ort_session, name)
            torch_type = TypeHelper.ort_type_to_torch_type(ort_type)
            output_buffers[name] = torch.empty(numpy.prod(shape), dtype=torch_type, device=device)
        return output_buffers

    @staticmethod
    def prepare_io_binding(
        ort_session,
        input_ids: torch.Tensor,
        position_ids: torch.Tensor,
        attention_mask: torch.Tensor,
        past: list[torch.Tensor],
        output_buffers,
        output_shapes,
        name_to_np_type=None,
    ):
        """Returnas IO binding object for a session."""
        if name_to_np_type is None:
            name_to_np_type = TypeHelper.get_io_numpy_type_map(ort_session)

        # Bind inputs and outputs to onnxruntime session
        io_binding = ort_session.io_binding()

        # Bind inputs
        assert input_ids.is_contiguous()
        io_binding.bind_input(
            "input_ids",
            input_ids.device.type,
            0,
            name_to_np_type["input_ids"],
            list(input_ids.size()),
            input_ids.data_ptr(),
        )

        if past is not None:
            for i, past_i in enumerate(past):
                assert past_i.is_contiguous()

                data_ptr = past_i.data_ptr()
                if data_ptr == 0:
                    # When past_sequence_length is 0, its data_ptr will be zero. IO Binding asserts that data_ptr shall not be zero.
                    # Here we workaround and pass data pointer of input_ids. Actual data is not used for past so it does not matter.
                    data_ptr = input_ids.data_ptr()

                io_binding.bind_input(
                    f"past_{i}",
                    past_i.device.type,
                    0,
                    name_to_np_type[f"past_{i}"],
                    list(past_i.size()),
                    data_ptr,
                )

        if attention_mask is not None:
            assert attention_mask.is_contiguous()
            io_binding.bind_input(
                "attention_mask",
                attention_mask.device.type,
                0,
                name_to_np_type["attention_mask"],
                list(attention_mask.size()),
                attention_mask.data_ptr(),
            )

        if position_ids is not None:
            assert position_ids.is_contiguous()
            io_binding.bind_input(
                "position_ids",
                position_ids.device.type,
                0,
                name_to_np_type["position_ids"],
                list(position_ids.size()),
                position_ids.data_ptr(),
            )

        # Bind outputs
        for output in ort_session.get_outputs():
            output_name = output.name
            output_buffer = output_buffers[output_name]
            logger.debug(f"{output_name} device type={output_buffer.device.type} shape={list(output_buffer.size())}")
            io_binding.bind_output(
                output_name,
                output_buffer.device.type,
                0,
                name_to_np_type[output_name],
                output_shapes[output_name],
                output_buffer.data_ptr(),
            )

        return io_binding

    @staticmethod
    def get_outputs_from_io_binding_buffer(ort_session, output_buffers, output_shapes, return_numpy=True):
        """Copy results to cpu. Returns a list of numpy array."""
        ort_outputs = []
        for output in ort_session.get_outputs():
            output_name = output.name
            buffer = output_buffers[output_name]
            shape = output_shapes[output_name]
            copy_tensor = buffer[0 : numpy.prod(shape)].reshape(shape).clone().detach()
            if return_numpy:
                ort_outputs.append(copy_tensor.cpu().numpy())
            else:
                ort_outputs.append(copy_tensor)
        return ort_outputs


class CudaSession:
    """Inference Session with IO Binding for ONNX Runtime CUDA or TensorRT provider"""

    def __init__(self, ort_session: InferenceSession, device: torch.device, enable_cuda_graph=False):
        self.ort_session = ort_session
        self.input_names = [input.name for input in self.ort_session.get_inputs()]
        self.output_names = [output.name for output in self.ort_session.get_outputs()]
        self.io_name_to_numpy_type = TypeHelper.get_io_numpy_type_map(self.ort_session)
        self.io_binding = self.ort_session.io_binding()
        self.enable_cuda_graph = enable_cuda_graph

        self.input_tensors = OrderedDict()
        self.output_tensors = OrderedDict()
        self.device = device

        # Pairs of input and output names that share the same buffer.
        self.buffer_sharing: dict[str, str] = {}

    def set_buffer_sharing(self, input_name: str, output_name: str):
        assert input_name in self.input_names
        assert output_name in self.output_names
        self.buffer_sharing[input_name] = output_name
        self.buffer_sharing[output_name] = input_name

    def __del__(self):
        del self.input_tensors
        del self.output_tensors
        del self.io_binding

    def bind_input_and_buffer_sharing(self, name: str, tensor: torch.Tensor):
        device_id = tensor.device.index if tensor.device.index is not None else 0
        tensor_shape = [1] if len(tensor.shape) == 0 else list(tensor.shape)

        self.io_binding.bind_input(
            name,
            tensor.device.type,
            device_id,
            self.io_name_to_numpy_type[name],
            tensor_shape,
            tensor.data_ptr(),
        )

        if name in self.buffer_sharing:
            self.io_binding.bind_output(
                self.buffer_sharing[name],
                tensor.device.type,
                device_id,
                self.io_name_to_numpy_type[name],
                tensor_shape,
                tensor.data_ptr(),
            )
            self.output_tensors[self.buffer_sharing[name]] = tensor

    def allocate_buffers(self, shape_dict: ShapeDict):
        """Allocate tensors for I/O Binding"""
        if self.enable_cuda_graph:
            for name, shape in shape_dict.items():
                if name in self.input_names:
                    # Reuse allocated buffer when the shape is same
                    if name in self.input_tensors:
                        if tuple(self.input_tensors[name].shape) == tuple(shape):
                            continue
                        raise RuntimeError("Expect static input shape for cuda graph")

                    numpy_dtype = self.io_name_to_numpy_type[name]
                    tensor = torch.empty(tuple(shape), dtype=TypeHelper.numpy_type_to_torch_type(numpy_dtype)).to(
                        device=self.device
                    )
                    self.input_tensors[name] = tensor
                    self.bind_input_and_buffer_sharing(name, tensor)

        for name, shape in shape_dict.items():
            if name in self.output_names:
                # Reuse allocated buffer when the shape is same
                if name in self.output_tensors and tuple(self.output_tensors[name].shape) == tuple(shape):
                    continue

                if name in self.buffer_sharing:
                    continue

                numpy_dtype = self.io_name_to_numpy_type[name]
                tensor = torch.empty(tuple(shape), dtype=TypeHelper.numpy_type_to_torch_type(numpy_dtype)).to(
                    device=self.device
                )
                self.output_tensors[name] = tensor

                self.io_binding.bind_output(
                    name,
                    tensor.device.type,
                    tensor.device.index if tensor.device.index is not None else 0,
                    numpy_dtype,
                    list(tensor.size()),
                    tensor.data_ptr(),
                )

    def infer(self, feed_dict: dict[str, torch.Tensor], run_options: RunOptions = None, synchronize: bool = True):
        """Bind input tensors and run inference"""
        for name, tensor in feed_dict.items():
            assert isinstance(tensor, torch.Tensor) and tensor.is_contiguous()
            if name in self.input_names:
                if self.enable_cuda_graph:
                    assert self.input_tensors[name].nelement() == tensor.nelement()
                    assert self.input_tensors[name].dtype == tensor.dtype
                    assert tensor.device.type == "cuda"
                    self.input_tensors[name].copy_(tensor)
                else:
                    self.bind_input_and_buffer_sharing(name, tensor)

        if synchronize:
            self.io_binding.synchronize_inputs()
            self.ort_session.run_with_iobinding(self.io_binding, run_options)
            self.io_binding.synchronize_outputs()
        else:
            self.ort_session.run_with_iobinding(self.io_binding, run_options)

        return self.output_tensors

    @staticmethod
    def get_cuda_provider_options(device_id: int, enable_cuda_graph: bool, stream: int = 0) -> dict[str, Any]:
        options = {
            "device_id": device_id,
            "arena_extend_strategy": "kSameAsRequested",
            "enable_cuda_graph": enable_cuda_graph,
        }

        # Stream is address of a CUDA stream. 0 means the default stream.
        if stream != 0:
            options["user_compute_stream"] = str(stream)

        return options


class GpuBinding(CudaSession):
    def __init__(
        self,
        ort_session: InferenceSession,
        device: torch.device,
        shape_dict: ShapeDict,
        enable_gpu_graph: bool = False,
        gpu_graph_id: int = -1,
        stream: int = 0,
        buffer_sharing: dict[str, str] | None = None,
    ):
        super().__init__(ort_session, device, enable_gpu_graph)
        if buffer_sharing:
            for input_name, output_name in buffer_sharing.items():
                self.set_buffer_sharing(input_name, output_name)

        self.allocate_buffers(shape_dict)
        self.gpu_graph_id = gpu_graph_id
        # For cuda graph, we need to keep a copy of shape_dict to check if the shape is same in inference later.
        self.shape_dict = copy.deepcopy(shape_dict) if enable_gpu_graph else None
        self.stream = stream
        # The gpu graph id of last run. It will be saved to image metadata.
        self.last_run_gpu_graph_id = None

    def get_run_options(self, disable_cuda_graph_in_run: bool = False) -> RunOptions:
        options = RunOptions()

        gpu_graph_id = -1 if disable_cuda_graph_in_run else self.gpu_graph_id

        options.add_run_config_entry("gpu_graph_id", str(gpu_graph_id))

        self.last_run_gpu_graph_id = gpu_graph_id

        return options

    def infer(self, feed_dict: dict[str, torch.Tensor], disable_cuda_graph_in_run: bool = False):
        run_options = self.get_run_options(disable_cuda_graph_in_run)

        if self.stream:
            run_options.add_run_config_entry("disable_synchronize_execution_providers", "1")

        return super().infer(feed_dict, run_options)


class GpuBindingManager:
    """A manager for I/O bindings that support multiple CUDA Graphs.
    One cuda graph is reused for same input shape. Automatically add a new cuda graph for new input shape.
    """

    def __init__(self, ort_session: InferenceSession, device: torch.device, stream: int = 0, max_cuda_graphs: int = 1):
        self.ort_session = ort_session
        self.device = device

        # Binding supports cuda graphs. For a binding, it is able to disable cuda graph for a specific run.
        self.graph_bindings = []

        # Binding for not using cuda graph.
        self.no_graph_binding = None

        self.stream = stream

        self.max_cuda_graphs = max_cuda_graphs

    def get_binding(
        self,
        shape_dict: ShapeDict,
        use_cuda_graph: bool = False,
        buffer_sharing: dict[str, str] | None = None,
    ) -> GpuBinding:
        for gpu_graph_binding in self.graph_bindings:
            # Found a cuda graph that captured with the same shape
            if gpu_graph_binding.shape_dict == shape_dict:
                return gpu_graph_binding

        # Reached the maximum number of cuda graphs. Return a binding without cuda graph.
        if len(self.graph_bindings) >= self.max_cuda_graphs or (not use_cuda_graph):
            if self.no_graph_binding is None:
                self.no_graph_binding = GpuBinding(
                    self.ort_session, self.device, shape_dict, stream=self.stream, buffer_sharing=buffer_sharing
                )
            else:
                self.no_graph_binding.allocate_buffers(shape_dict)
            return self.no_graph_binding

        # This is a new input shape, create a new cuda graph
        gpu_graph_binding = GpuBinding(
            self.ort_session,
            self.device,
            shape_dict,
            enable_gpu_graph=True,
            gpu_graph_id=len(self.graph_bindings),
            stream=self.stream,
            buffer_sharing=buffer_sharing,
        )
        self.graph_bindings.append(gpu_graph_binding)
        return gpu_graph_binding



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
# --------------------------------------------------------------------------
# isort: skip_file
from onnxruntime.capi._pybind_state import (
    PropagateCastOpsStrategy,
    TrainingParameters,
    is_ortmodule_available,
)

# Options need to be imported before `ORTTrainer`.
from . import amp, artifacts, optim

__all__ = [
    "PropagateCastOpsStrategy",
    "TrainingParameters",
    "amp",
    "artifacts",
    "is_ortmodule_available",
    "optim",
]

try:
    if is_ortmodule_available():
        from .ortmodule import ORTModule

        __all__ += ["ORTModule"]
except ImportError:
    # That is OK iff this is not a ORTModule training package
    pass



import coremltools as ct
import numpy as np
from coremltools.converters.mil import Builder as mb

target = ct.target.iOS15

a_shape = (1, 1, 3, 3)


@mb.program(
    input_specs=[mb.TensorSpec(shape=a_shape), mb.TensorSpec(shape=a_shape), mb.TensorSpec(shape=a_shape)],
    opset_version=target,
)
def prog(x, y, z):
    axis = mb.const(val=1)
    interleave = mb.const(val=False)
    z = mb.concat(values=(x, y, z), axis=axis, interleave=interleave)
    return z


print(prog)

# Convert to ML program
m = ct.convert(prog, minimum_deployment_target=target, compute_precision=ct.precision.FLOAT32)

x = np.random.rand(*a_shape)
y = np.random.rand(*a_shape)
z = np.random.rand(*a_shape)

# spec = m.get_spec()
# print(spec)

print(m.predict({"x": x, "y": y, "z": z}))



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

from onnxruntime.training.api.checkpoint_state import CheckpointState
from onnxruntime.training.api.lr_scheduler import LinearLRScheduler
from onnxruntime.training.api.module import Module
from onnxruntime.training.api.optimizer import Optimizer

__all__ = [
    "CheckpointState",
    "LinearLRScheduler",
    "Module",
    "Optimizer",
]



import torch
import torch.onnx.symbolic_helper
import torch.onnx.utils


def _export_jit_graph_to_onnx_model_proto(graph: torch._C.Graph, operator_export_type: int):
    from torch.onnx.symbolic_helper import (  # noqa: F401
        _set_onnx_shape_inference,
        _set_operator_export_type,
        _set_opset_version,
    )

    _set_onnx_shape_inference(True)
    _set_operator_export_type(operator_export_type)
    torch._C._jit_pass_run_decompositions(graph)
    graph = torch.onnx.utils._optimize_graph(graph, operator_export_type, params_dict={})
    proto, _, _, _ = graph._export_onnx(
        {},
        torch.onnx._globals.GLOBALS.export_onnx_opset_version,
        {},
        False,
        operator_export_type,
        False,
        False,
        {},
        True,
        "",
        {},
    )
    return proto



#!/usr/bin/env python3
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

import os
import sys

# This is a script to validate NPM packages.
# If package version, publish tag and filename does not fulfill the requirement, an error will raise.

# arg.1 - Folder of extracted artifact "onnxruntime-node" for node.js binding
ort_node_pkg_dir = sys.argv[1]
# arg.2 - Folder of extracted artifact "onnxruntime-web" for web
ort_web_pkg_dir = sys.argv[2]
# arg.3 - Folder of extracted artifact "onnxruntime-react-native" for react native
ort_react_native_pkg_dir = sys.argv[3]
# arg.4 - source branch, eg. "refs/heads/master"
source_branch = sys.argv[4]
# arg.5 - NPM tag, eg. "", "dev", "latest", "rc"
tag = sys.argv[5]

# print out command line parameters
print("====== argv ======")
print("ort_node_pkg_dir:", ort_node_pkg_dir)
print("ort_web_pkg_dir:", ort_web_pkg_dir)
print("ort_react_native_pkg_dir:", ort_react_native_pkg_dir)
print("source_branch:", source_branch)
print("tag:", tag)

# check release flags from environment variables
RELEASE_NODE = os.environ.get("RELEASE_NODE", "") == "1"
RELEASE_WEB = os.environ.get("RELEASE_WEB", "") == "1"
RELEASE_REACT_NATIVE = os.environ.get("RELEASE_REACT_NATIVE", "") == "1"

# print ouf release flags
print("====== flags ======")
print("RELEASE_NODE:", RELEASE_NODE)
print("RELEASE_WEB:", RELEASE_WEB)
print("RELEASE_REACT_NATIVE:", RELEASE_REACT_NATIVE)

if not RELEASE_NODE and not RELEASE_WEB and not RELEASE_REACT_NATIVE:
    raise Exception("not releasing any package. exiting.")

count_ort_node_common_tgz = 0
count_ort_node_tgz = 0
ort_node_common_ver = ""
ort_node_ver = ""

for file in os.listdir(ort_node_pkg_dir):
    if file.startswith("onnxruntime-common-") and file.endswith(".tgz"):
        ort_node_common_ver = file[19:-4]
        count_ort_node_common_tgz += 1
    if file.startswith("onnxruntime-node-") and file.endswith(".tgz"):
        ort_node_ver = file[17:-4]
        count_ort_node_tgz += 1

count_ort_web_common_tgz = 0
count_ort_web_tgz = 0
ort_web_common_ver = ""
ort_web_ver = ""

for file in os.listdir(ort_web_pkg_dir):
    if file.startswith("onnxruntime-common-") and file.endswith(".tgz"):
        ort_web_common_ver = file[19:-4]
        count_ort_web_common_tgz += 1
    if file.startswith("onnxruntime-web-") and file.endswith(".tgz"):
        ort_web_ver = file[16:-4]
        count_ort_web_tgz += 1

count_ort_react_native_common_tgz = 0
count_ort_react_native_tgz = 0
ort_react_native_common_ver = ""
ort_react_native_ver = ""

for file in os.listdir(ort_react_native_pkg_dir):
    if file.startswith("onnxruntime-common-") and file.endswith(".tgz"):
        ort_react_native_common_ver = file[19:-4]
        count_ort_react_native_common_tgz += 1
    if file.startswith("onnxruntime-react-native-") and file.endswith(".tgz"):
        ort_react_native_ver = file[25:-4]
        count_ort_react_native_tgz += 1

if count_ort_node_common_tgz >= 2:
    raise Exception("expect at most 1 package file for onnxruntime-common in onnxruntime-node folder")
if count_ort_web_common_tgz >= 2:
    raise Exception("expect at most 1 package file for onnxruntime-common in onnxruntime-web folder")
if count_ort_react_native_common_tgz >= 2:
    raise Exception("expect at most 1 package file for onnxruntime-common in onnxruntime-react-native folder")

if RELEASE_NODE and RELEASE_WEB and count_ort_node_common_tgz != count_ort_web_common_tgz:
    raise Exception("inconsistent package number for onnxruntime-common (onnxruntime-node/onnxruntime-web)")
if RELEASE_NODE and RELEASE_REACT_NATIVE and count_ort_node_common_tgz != count_ort_react_native_common_tgz:
    raise Exception("inconsistent package number for onnxruntime-common (onnxruntime-node/onnxruntime-react-native)")
if RELEASE_WEB and RELEASE_REACT_NATIVE and count_ort_web_common_tgz != count_ort_react_native_common_tgz:
    raise Exception("inconsistent package number for onnxruntime-common (onnxruntime-web/onnxruntime-react-native)")

if RELEASE_NODE and RELEASE_WEB and ort_node_common_ver != ort_web_common_ver:
    raise Exception("inconsistent version number for onnxruntime-common (onnxruntime-node/onnxruntime-web)")
if RELEASE_NODE and RELEASE_REACT_NATIVE and ort_node_common_ver != ort_react_native_common_ver:
    raise Exception("inconsistent version number for onnxruntime-common (onnxruntime-node/onnxruntime-react-native)")
if RELEASE_WEB and RELEASE_REACT_NATIVE and ort_web_common_ver != ort_react_native_common_ver:
    raise Exception("inconsistent version number for onnxruntime-common (onnxruntime-web/onnxruntime-react-native)")

ort_common_ver = (
    ort_node_common_ver if RELEASE_NODE else (ort_web_common_ver if RELEASE_WEB else ort_react_native_common_ver)
)

ort_common_from = "" if not ort_common_ver else ("node" if RELEASE_NODE else ("web" if RELEASE_WEB else "react-native"))

print("====== output environment variables ======")
print(f"##vso[task.setvariable variable=ORT_COMMON_FROM]{ort_common_from}")

if tag == "latest" or tag == "" or tag == "rc":
    if not RELEASE_NODE or not RELEASE_WEB or not RELEASE_REACT_NATIVE:
        raise Exception("@latest or @rc build must release all packages (node, web, react-native)")
    if count_ort_node_common_tgz != 1:
        raise Exception("expect one package file for onnxruntime-common for release build")

if count_ort_node_tgz != 1:
    raise Exception("expect one package file for onnxruntime-node")
if count_ort_web_tgz != 1:
    raise Exception("expect one package file for onnxruntime-web")
if count_ort_react_native_tgz != 1:
    raise Exception("expect one package file for onnxruntime-react-native")
if RELEASE_NODE and RELEASE_WEB and ort_node_ver != ort_web_ver:
    raise Exception("version number is different for onnxruntime-node and onnxruntime-web")
if RELEASE_NODE and RELEASE_REACT_NATIVE and ort_node_ver != ort_react_native_ver:
    raise Exception("version number is different for onnxruntime-node and onnxruntime-react-native")
if RELEASE_WEB and RELEASE_REACT_NATIVE and ort_web_ver != ort_react_native_ver:
    raise Exception("version number is different for onnxruntime-web and onnxruntime-react-native")

print("====== validated versions ======")
print(f"source_branch={source_branch}")
print(f"tag={tag}")
print(f"ort_common_ver={ort_common_ver}")
print(f"ort_node_ver={ort_node_ver}")
print(f"ort_web_ver={ort_web_ver}")
print(f"ort_react_native_ver={ort_react_native_ver}")

if tag == "latest" or tag == "":
    print("Publishing @latest ...")
    if not source_branch.startswith("refs/heads/rel-"):
        raise Exception('@latest build must publish from source branch "refs/heads/rel-*"')
    if (
        "-" in ort_common_ver.replace("-rev", "")
        or "-" in ort_web_ver.replace("-rev", "")
        or "-" in ort_react_native_ver.replace("-rev", "")
    ):
        raise Exception('@latest build version cannot contain "-" (unless -rev)')
if tag == "rc":
    print("Publishing @rc ...")
    if not source_branch.startswith("refs/heads/rel-"):
        raise Exception('@rc build must publish from source branch "refs/heads/rel-*"')
    if "-rc" not in ort_web_ver:
        raise Exception('@rc build version should contain "-rc"')
    if "-rc" not in ort_react_native_ver:
        raise Exception('@rc build version should contain "-rc"')

if (
    "-" not in ort_common_ver.replace("-rev", "")
    and "-" not in ort_web_ver.replace("-rev", "")
    and "-" not in ort_react_native_ver.replace("-rev", "")
    and "+" not in ort_common_ver.replace("-rev", "")
    and "+" not in ort_web_ver.replace("-rev", "")
    and "+" not in ort_react_native_ver.replace("-rev", "")
):
    if tag != "latest" and tag != "":
        raise Exception("default version without decorator can only be published in @latest tag")



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

import sys


def is_windows():
    return sys.platform.startswith("win")


def is_macOS():  # noqa: N802
    return sys.platform.startswith("darwin")


def is_linux():
    return sys.platform.startswith("linux")



from setuptools import setup
from torch.utils.cpp_extension import BuildExtension, CppExtension

setup(
    name="longformer_attention",
    ext_modules=[
        CppExtension(
            name="longformer_attention",
            sources=["longformer_attention.cpp"],
            include_dirs=[],
            extra_compile_args=["-g"],
        )
    ],
    cmdclass={"build_ext": BuildExtension},
)



#!/usr/bin/env python
# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import unittest

import numpy as np
import onnx
from onnx import TensorProto, helper
from op_test_utils import TestDataFeeds, check_model_correctness, check_op_type_count, check_qtype_by_node_type

from onnxruntime.quantization import QuantFormat, QuantType, quantize_static


class TestOpSqueezeUnsqueeze(unittest.TestCase):
    def input_feeds(self, n, name2shape):
        input_data_list = []
        for _i in range(n):
            inputs = {}
            for name, shape in name2shape.items():
                inputs.update({name: np.random.randint(-1, 2, shape).astype(np.float32)})
            input_data_list.extend([inputs])
        dr = TestDataFeeds(input_data_list)
        return dr

    def construct_model_conv_squeezes(
        self,
        output_model_path,
        conv_input_shape,
        conv_weight_shape,
        conv_output_shape,
        opset=13,
    ):
        #             (input)
        #            /   |     \
        #         Conv1 conv2    conv3
        #           |     |         |
        #       Squeeze1 Squeeze2   |
        #           \      /        |
        #             add1          |
        #               |           |
        #              Unsqueeze    |
        #                      \    |
        #                       add2
        #                         |
        #                      (output)
        input_tensor = helper.make_tensor_value_info("input", TensorProto.FLOAT, conv_input_shape)

        conv1_weight_arr = np.random.randint(-1, 2, conv_weight_shape).astype(np.float32)
        conv1_weight_initializer = onnx.numpy_helper.from_array(conv1_weight_arr, name="conv1_weight")
        conv1_node = onnx.helper.make_node("Conv", ["input", "conv1_weight"], ["conv1_output"], name="conv1_node")

        conv2_weight_arr = np.random.randint(-1, 2, conv_weight_shape).astype(np.float32)
        conv2_weight_initializer = onnx.numpy_helper.from_array(conv2_weight_arr, name="conv2_weight")
        conv2_node = onnx.helper.make_node("Conv", ["input", "conv2_weight"], ["conv2_output"], name="conv2_node")

        conv3_weight_arr = np.random.randint(-1, 2, conv_weight_shape).astype(np.float32)
        conv3_weight_initializer = onnx.numpy_helper.from_array(conv3_weight_arr, name="conv3_weight")
        conv3_node = onnx.helper.make_node("Conv", ["input", "conv3_weight"], ["conv3_output"], name="conv3_node")

        if opset >= 13:
            squeeze_axes_initializer = onnx.numpy_helper.from_array(np.array([0], dtype=np.int64), name="squeeze_axes")
            squeeze1_node = helper.make_node(
                "Squeeze",
                ["conv1_output", "squeeze_axes"],
                ["squeeze1_output"],
                name="suqeeze1_node",
            )
            squeeze2_node = helper.make_node(
                "Squeeze",
                ["conv2_output", "squeeze_axes"],
                ["squeeze2_output"],
                name="suqeeze2_node",
            )
        else:
            squeeze1_node = helper.make_node(
                "Squeeze",
                ["conv1_output"],
                ["squeeze1_output"],
                name="suqeeze1_node",
                axes=[0],
            )
            squeeze2_node = helper.make_node(
                "Squeeze",
                ["conv2_output"],
                ["squeeze2_output"],
                name="suqeeze2_node",
                axes=[0],
            )

        add1_node = helper.make_node(
            "Add",
            ["squeeze1_output", "squeeze2_output"],
            ["add1_output"],
            name="add1_node",
        )
        if opset >= 13:
            unsqueeze_node = helper.make_node(
                "Unsqueeze",
                ["add1_output", "squeeze_axes"],
                ["unsqueeze_output"],
                name="unsqueeze_node",
            )
        else:
            unsqueeze_node = helper.make_node(
                "Unsqueeze",
                ["add1_output"],
                ["unsqueeze_output"],
                name="unsqueeze_node",
                axes=[0],
            )

        output_tensor = helper.make_tensor_value_info("output", TensorProto.FLOAT, conv_output_shape)
        add2_node = helper.make_node("Add", ["unsqueeze_output", "conv3_output"], ["output"], name="add2_node")

        initializers = [
            conv1_weight_initializer,
            conv2_weight_initializer,
            conv3_weight_initializer,
        ]
        if opset >= 13:
            initializers.append(squeeze_axes_initializer)
        graph = helper.make_graph(
            [
                conv1_node,
                conv2_node,
                conv3_node,
                squeeze1_node,
                squeeze2_node,
                add1_node,
                unsqueeze_node,
                add2_node,
            ],
            "TestOpSuqeezes_test_model",
            [input_tensor],
            [output_tensor],
            initializer=initializers,
        )
        model = helper.make_model(graph, opset_imports=[helper.make_opsetid("", opset)])
        model.ir_version = 7  # use stable onnx ir version
        onnx.save(model, output_model_path)

    def run_quantize_squeezes_of_opset(
        self,
        opset=13,
        activation_type=QuantType.QUInt8,
        weight_type=QuantType.QUInt8,
        extra_options={},  # noqa: B006
    ):
        np.random.seed(1)

        model_fp32_path = f"squeezes_opset{opset}_fp32.onnx"
        self.construct_model_conv_squeezes(model_fp32_path, [1, 2, 26, 42], [3, 2, 3, 3], [1, 3, 24, 40], opset=opset)

        activation_proto_qtype = TensorProto.UINT8 if activation_type == QuantType.QUInt8 else TensorProto.INT8
        activation_type_str = "u8" if (activation_type == QuantType.QUInt8) else "s8"
        weight_type_str = "u8" if (weight_type == QuantType.QUInt8) else "s8"
        model_uint8_path = f"squeezes_opset{opset}_{activation_type_str}{weight_type_str}.onnx"
        model_uint8_qdq_path = f"squeezes_opset{opset}_{activation_type_str}{weight_type_str}_qdq.onnx"

        # Verify QOperator mode
        data_reader = self.input_feeds(1, {"input": [1, 2, 26, 42]})
        quantize_static(
            model_fp32_path,
            model_uint8_path,
            data_reader,
            quant_format=QuantFormat.QOperator,
            activation_type=activation_type,
            weight_type=weight_type,
            extra_options=extra_options,
        )

        # make sure squeezes become xint8 operator, its input name could tell that
        qnode_counts = {"QuantizeLinear": 1, "DequantizeLinear": 1}
        check_op_type_count(self, model_uint8_path, **qnode_counts)
        qnode_io_qtypes = {
            "QuantizeLinear": [
                ["i", 2, activation_proto_qtype],
                ["o", 0, activation_proto_qtype],
            ]
        }
        qnode_io_qtypes.update({"DequantizeLinear": [["i", 2, activation_proto_qtype]]})
        check_qtype_by_node_type(self, model_uint8_path, qnode_io_qtypes)
        data_reader.rewind()
        check_model_correctness(
            self,
            model_fp32_path,
            model_uint8_path,
            data_reader.get_next(),
            rtol=0.01,
            atol=0.5,
        )

        # Verify QDQ mode
        data_reader.rewind()
        quantize_static(
            model_fp32_path,
            model_uint8_qdq_path,
            data_reader,
            quant_format=QuantFormat.QDQ,
            activation_type=activation_type,
            weight_type=weight_type,
            extra_options=extra_options,
        )
        qdqnode_counts = {"Conv": 3, "QuantizeLinear": 9, "DequantizeLinear": 12}
        check_op_type_count(self, model_uint8_qdq_path, **qdqnode_counts)
        qnode_io_qtypes = {
            "QuantizeLinear": [
                ["i", 2, activation_proto_qtype],
                ["o", 0, activation_proto_qtype],
            ]
        }
        check_qtype_by_node_type(self, model_uint8_qdq_path, qnode_io_qtypes)
        data_reader.rewind()
        check_model_correctness(
            self,
            model_fp32_path,
            model_uint8_qdq_path,
            data_reader.get_next(),
            rtol=0.01,
            atol=0.5,
        )

    def test_quantize_squeeze_unsqueeze(self):
        self.run_quantize_squeezes_of_opset(11)
        self.run_quantize_squeezes_of_opset(13)

    def test_quantize_squeeze_unsqueeze_s8s8(self):
        self.run_quantize_squeezes_of_opset(
            11,
            QuantType.QInt8,
            QuantType.QInt8,
            extra_options={"ActivationSymmetric": True},
        )
        self.run_quantize_squeezes_of_opset(
            13,
            QuantType.QInt8,
            QuantType.QInt8,
            extra_options={"ActivationSymmetric": True},
        )


if __name__ == "__main__":
    unittest.main()



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

import unittest
from collections.abc import Sequence
from copy import deepcopy

import numpy as np
from onnx import ModelProto, NodeProto, TensorProto, ValueInfoProto, checker, helper

import onnxruntime as ort


def make_vi_like(vi: ValueInfoProto, name: str) -> ValueInfoProto:
    """Makes a copy of `vi` with a new name."""
    new_vi = deepcopy(vi)
    new_vi.name = name
    return new_vi


def make_optional_tensor_value_info(name: str, elem_type: int, shape: Sequence[int]) -> ValueInfoProto:
    """Makes a `ValueInfoProto` with optional type."""
    tensor_type_proto = helper.make_tensor_type_proto(
        elem_type=elem_type,
        shape=shape,
    )
    opt_type_proto = helper.make_optional_type_proto(tensor_type_proto)

    vi = helper.make_tensor_value_info(name, elem_type, shape)
    vi.type.CopyFrom(opt_type_proto)
    return vi


def make_optional_vi(vi: ValueInfoProto, name: str | None = None) -> ValueInfoProto:
    """Makes a copy of `vi` with optional type."""
    name = name or vi.name + ".opt"
    vi_type = vi.type.tensor_type
    vi_shape = [d.dim_param if d.dim_param else d.dim_value for d in vi_type.shape.dim]
    opt_vi = make_optional_tensor_value_info(name, vi_type.elem_type, vi_shape)
    return opt_vi


def make_const(vi: ValueInfoProto, name: str, value: int = 0) -> tuple[ValueInfoProto, NodeProto, TensorProto]:
    """Creates a constant 1D tensor from `vi`."""
    const_vi = make_vi_like(vi, name)
    const_shape = [d.dim_value for d in vi.type.tensor_type.shape.dim]
    const_shape_tensor = helper.make_tensor(f"{name}.shape", TensorProto.INT64, [len(const_shape)], const_shape)
    const_fill = helper.make_tensor(f"{name}.const.value", const_vi.type.tensor_type.elem_type, [1], [value])
    const_node = helper.make_node(
        "ConstantOfShape",
        inputs=[const_shape_tensor.name],
        outputs=[const_vi.name],
        name=f"ConstantOfShape.{name}",
        value=const_fill,
    )
    return const_vi, const_node, const_shape_tensor


# This is a three-layer nested control flow ops model.
# The innermost subgraphs have the outer scope values that are the inputs, x2 and x3, of the top-level graph.
def make_opt_nested_greater_or_equal() -> ModelProto:
    """
    Creates a nested graph with (`optional(x1)`, `x2`, x3`) tensor inputs.

    `x3` is similar to an optional input with default value of -1.
    """
    # Inputs/outputs
    x1_vi = helper.make_tensor_value_info("x1", TensorProto.FLOAT, [1, 2])
    x2_vi = helper.make_tensor_value_info("x2", TensorProto.FLOAT, [1, 2])
    x3_vi = helper.make_tensor_value_info("x3", TensorProto.FLOAT, [1])
    y_vi = helper.make_tensor_value_info("y", TensorProto.FLOAT, [1, 2])
    opt_x1_vi = make_optional_vi(x1_vi, name="x1.opt")

    # Add `x1` and `x2` subgraph
    y1_vi = make_vi_like(y_vi, "x1.add.x2")
    input_get_elem_node = helper.make_node(
        "OptionalGetElement",
        inputs=[opt_x1_vi.name],
        outputs=[x1_vi.name],
        name="OptionalGetElement.Input",
    )
    add_node = helper.make_node("Add", inputs=[x1_vi.name, x2_vi.name], outputs=[y1_vi.name], name="Add_Op")
    add_x1_x2_subgraph = helper.make_graph(
        [input_get_elem_node, add_node],
        name="add-x1-x2-subgraph",
        inputs=[],
        outputs=[y1_vi],
        value_info=[opt_x1_vi, x1_vi, x2_vi],
    )

    # Add `x2` and const subgraph
    y2_vi = make_vi_like(y_vi, "x2.add.const")
    const_vi, const_node, const_shape_tensor = make_const(x1_vi, "x1.const", value=1)
    add_const_node = helper.make_node(
        "Add",
        inputs=[const_vi.name, x2_vi.name],
        outputs=[y2_vi.name],
        name="Add_Const",
    )
    add_x2_const_subgraph = helper.make_graph(
        [const_node, add_const_node],
        name="add-x2-const-subgraph",
        inputs=[],
        outputs=[y2_vi],
        value_info=[const_vi, x2_vi],
        initializer=[const_shape_tensor],
    )

    # Add `x3` and const subgraph
    add_const_out_vi = make_vi_like(x3_vi, "out.1")
    y3_vi = make_vi_like(y_vi, "x3.add.const")
    const_vi, const_node, const_shape_tensor = make_const(x3_vi, "x3.const", value=2)
    add_const_node = helper.make_node(
        "Add",
        inputs=[const_vi.name, x3_vi.name],
        outputs=[add_const_out_vi.name],
        name="Add_Const.1",
    )
    expand_shape = helper.make_tensor(f"{add_const_out_vi}.shape", TensorProto.INT64, [2], [1, 2])
    expand_node = helper.make_node(
        "Expand",
        inputs=[add_const_out_vi.name, expand_shape.name],
        outputs=[y3_vi.name],
        name="Expand.out",
    )
    add_x3_const_subgraph = helper.make_graph(
        [const_node, add_const_node, expand_node],
        name="add-x3-const-subgraph",
        inputs=[],
        outputs=[y3_vi],
        value_info=[x3_vi, const_vi, add_const_out_vi],
        initializer=[const_shape_tensor, expand_shape],
    )

    # Subgraph flow based on `x3` value
    y3_if_vi = make_vi_like(y_vi, "x3.if.out")
    x3_eq_vi, x3_const_node, x3_const_shape_tensor = make_const(x3_vi, "x3.equal", value=0)
    x3_ge_vi = helper.make_tensor_value_info(
        "x3_ge",
        TensorProto.BOOL,
        shape=[1],
    )
    x3_ge_node = helper.make_node(
        "GreaterOrEqual",
        inputs=[x3_vi.name, x3_eq_vi.name],
        outputs=[x3_ge_vi.name],
        name="GreaterOrEqual.Target",
    )
    x3_has_elem_vi = helper.make_tensor_value_info(
        "x3_has_elem",
        TensorProto.BOOL,
        shape=[],  # scalar
    )
    x3_has_elem_node = helper.make_node(
        "Squeeze",
        inputs=[x3_ge_vi.name],
        outputs=[x3_has_elem_vi.name],
        name="Squeeze.x3",
    )
    if_input_node = helper.make_node(
        "If",
        inputs=[x3_has_elem_vi.name],  # condition
        outputs=[y3_if_vi.name],
        name="If.OptionalHasElement.x3",
        then_branch=add_x3_const_subgraph,
        else_branch=add_x2_const_subgraph,
    )
    x3_subgraph = helper.make_graph(
        [x3_const_node, x3_ge_node, x3_has_elem_node, if_input_node],
        name="x3-subgraph",
        inputs=[],
        outputs=[y3_if_vi],
        value_info=[x3_vi, x3_eq_vi, x3_ge_vi, x3_has_elem_vi],
        initializer=[x3_const_shape_tensor],
    )

    # Construct main graph
    x1_has_elem_vi = helper.make_tensor_value_info(
        "x1_has_elem",
        TensorProto.BOOL,
        shape=[],  # scalar
    )
    x1_has_elem_node = helper.make_node(
        "OptionalHasElement",
        inputs=[opt_x1_vi.name],
        outputs=[x1_has_elem_vi.name],
        name="OptionalHasElement.x1",
    )
    if_input_node = helper.make_node(
        "If",
        inputs=[x1_has_elem_vi.name],  # condition
        outputs=[y_vi.name],
        name="If.OptionalHasElement.x1",
        then_branch=add_x1_x2_subgraph,
        else_branch=x3_subgraph,
    )
    graph = helper.make_graph(
        [x1_has_elem_node, if_input_node],
        "opt-graph",
        [opt_x1_vi, x2_vi, x3_vi],
        [y_vi],
        value_info=[x1_has_elem_vi],
    )

    m = helper.make_model(
        graph,
        opset_imports=[
            helper.make_opsetid("", 15),
        ],
    )

    checker.check_model(m, full_check=True)

    return m


def test_nested_optional_greater_or_equal(use_trt: bool = False) -> None:
    m = make_opt_nested_greater_or_equal()

    providers = ["CUDAExecutionProvider"]
    if use_trt:
        providers.insert(0, "TensorrtExecutionProvider")
    session = ort.InferenceSession(
        m.SerializeToString(),
        providers=providers,
    )

    x1_name, x2_name, x3_name = (i.name for i in m.graph.input)
    session.run(
        [m.graph.output[0].name],
        {
            x1_name: None,
            x2_name: np.ones((1, 2), dtype=np.float32),
            x3_name: np.array([-1], dtype=np.float32),
        },
    )

    return


# ORT has a similar unit test Test3LayerNestedSubgraph where this 3-layer nested graph consumes the same initializer in different subgraphs.
# However, this unit test is slightly different. This is also a 3-layer nested graph but consumes the outer scope values (which are the inputs
# of the top-level graph) in different subgraphs.
class TestNestedControlFlowOpsGraph(unittest.TestCase):
    # We currently only test CUDA/TRT EP due to users only raise this issue when using CUDA/TRT EP.
    @unittest.skipIf(
        "TensorrtExecutionProvider" not in ort.get_available_providers()
        and "CUDAExecutionProvider" not in ort.get_available_providers(),
        reason="Test CUDA/TRT EP only",
    )
    def test_3_level_control_flow_ops_graph(self):
        if "CUDAExecutionProvider" in ort.get_available_providers():
            test_nested_optional_greater_or_equal(use_trt=False)
        if "TensorrtExecutionProvider" in ort.get_available_providers():
            test_nested_optional_greater_or_equal(use_trt=True)


if __name__ == "__main__":
    unittest.main(module=__name__, buffer=True)



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

from onnxruntime.training.onnxblock.loss.loss import BCEWithLogitsLoss, CrossEntropyLoss, L1Loss, MSELoss

__all__ = ["BCEWithLogitsLoss", "CrossEntropyLoss", "L1Loss", "MSELoss"]



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
#
# Test reference implementation and model for ONNX Runtime conrtib op trilu

import unittest

import numpy as np
import onnx
from onnx_contrib_ops_helper import expect


def triu_reference_implementation(x, k=0):
    return np.triu(x, k)


def tril_reference_implementation(x, k=0):
    return np.tril(x, k)


class ONNXReferenceImplementationTest(unittest.TestCase):
    def test_triu(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x"],
            outputs=["y"],
            domain="com.microsoft",
        )

        x = np.random.randn(3, 4, 5).astype(np.float32)
        y = triu_reference_implementation(x)
        expect(node, inputs=[x], outputs=[y], name="test_triu")

    def test_triu_neg(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            domain="com.microsoft",
        )

        x = np.random.randn(3, 4, 5).astype(np.float32)
        k = np.array([-1]).astype(np.int64)
        y = triu_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_neg")

    def test_triu_out_neg(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            domain="com.microsoft",
        )

        x = np.random.randn(3, 4, 5).astype(np.float32)
        k = np.array([-7]).astype(np.int64)
        y = triu_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_out_neg")

    def test_triu_pos(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            domain="com.microsoft",
        )

        x = np.random.randn(3, 4, 5).astype(np.float32)
        k = np.array([2]).astype(np.int64)
        y = triu_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_pos")

    def test_triu_out_pos(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            domain="com.microsoft",
        )

        x = np.random.randn(3, 4, 5).astype(np.float32)
        k = np.array([6]).astype(np.int64)
        y = triu_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_out_pos")

    def test_triu_square(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x"],
            outputs=["y"],
            domain="com.microsoft",
        )

        x = np.random.randn(3, 5, 5).astype(np.float32)
        y = triu_reference_implementation(x)
        expect(node, inputs=[x], outputs=[y], name="test_triu_square")

    def test_triu_square_neg(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            domain="com.microsoft",
        )

        x = np.random.randn(3, 5, 5).astype(np.float32)
        k = np.array([-1]).astype(np.int64)
        y = triu_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_square_neg")

    def test_triu_one_row_neg(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            domain="com.microsoft",
        )

        x = np.random.randn(3, 1, 5).astype(np.float32)
        k = np.array([-7]).astype(np.int64)
        y = triu_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_one_row_neg")

    def test_triu_square_pos(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            domain="com.microsoft",
        )

        x = np.random.randn(3, 5, 5).astype(np.float32)
        k = np.array([2]).astype(np.int64)
        y = triu_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_square_pos")

    def test_triu_zero(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            domain="com.microsoft",
        )

        x = np.random.randn(3, 0, 5).astype(np.float32)
        k = np.array([6]).astype(np.int64)
        y = triu_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_zero")

    def test_tril(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x"],
            outputs=["y"],
            upper=0,
            domain="com.microsoft",
        )

        x = np.random.randn(3, 4, 5).astype(np.float32)
        y = tril_reference_implementation(x)
        expect(node, inputs=[x], outputs=[y], name="test_tril")

    def test_tril_neg(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
            domain="com.microsoft",
        )

        x = np.random.randn(3, 4, 5).astype(np.float32)
        k = np.array([-1]).astype(np.int64)
        y = tril_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_neg")

    def test_tril_out_neg(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
            domain="com.microsoft",
        )

        x = np.random.randn(3, 4, 5).astype(np.float32)
        k = np.array([-7]).astype(np.int64)
        y = tril_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_out_neg")

    def test_tril_pos(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
            domain="com.microsoft",
        )

        x = np.random.randn(3, 4, 5).astype(np.float32)
        k = np.array([2]).astype(np.int64)
        y = tril_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_pos")

    def test_tril_out_pos(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
            domain="com.microsoft",
        )

        x = np.random.randn(3, 4, 5).astype(np.float32)
        k = np.array([6]).astype(np.int64)
        y = tril_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_out_pos")

    def test_tril_square(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x"],
            outputs=["y"],
            upper=0,
            domain="com.microsoft",
        )

        x = np.random.randn(3, 5, 5).astype(np.float32)
        y = tril_reference_implementation(x)
        expect(node, inputs=[x], outputs=[y], name="test_tril_square")

    def test_tril_square_neg(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
            domain="com.microsoft",
        )

        x = np.random.randn(3, 5, 5).astype(np.float32)
        k = np.array([-1]).astype(np.int64)
        y = tril_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_square_neg")

    def test_tril_one_row_neg(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
            domain="com.microsoft",
        )

        x = np.random.randn(3, 1, 5).astype(np.float32)
        k = np.array([-7]).astype(np.int64)
        y = tril_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_one_row_neg")

    def test_tril_square_pos(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
            domain="com.microsoft",
        )

        x = np.random.randn(3, 5, 5).astype(np.float32)
        k = np.array([2]).astype(np.int64)
        y = tril_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_square_pos")

    def test_tril_zero(self):
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
            domain="com.microsoft",
        )

        x = np.random.randn(3, 0, 5).astype(np.float32)
        k = np.array([6]).astype(np.int64)
        y = tril_reference_implementation(x, k)
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_zero")


if __name__ == "__main__":
    unittest.main(module=__name__, buffer=True)



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

from onnxruntime.training.onnxblock.optim.optim import SGD, AdamW, ClipGradNorm

__all__ = ["SGD", "AdamW", "ClipGradNorm"]



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

"""
.. _l-example-common-error:

Common errors with onnxruntime
==============================

This example looks into several common situations
in which *onnxruntime* does not return the model
prediction but raises an exception instead.
It starts by loading the model trained in example
:ref:`l-logreg-example` which produced a logistic regression
trained on *Iris* datasets. The model takes
a vector of dimension 2 and returns a class among three.
"""

import numpy

import onnxruntime as rt
from onnxruntime.capi.onnxruntime_pybind11_state import InvalidArgument
from onnxruntime.datasets import get_example

example2 = get_example("logreg_iris.onnx")
sess = rt.InferenceSession(example2, providers=rt.get_available_providers())

input_name = sess.get_inputs()[0].name
output_name = sess.get_outputs()[0].name

#############################
# The first example fails due to *bad types*.
# *onnxruntime* only expects single floats (4 bytes)
# and cannot handle any other kind of floats.

try:
    x = numpy.array([[1.0, 2.0], [3.0, 4.0], [5.0, 6.0]], dtype=numpy.float64)
    sess.run([output_name], {input_name: x})
except Exception as e:
    print("Unexpected type")
    print(f"{type(e)}: {e}")

#########################
# The model fails to return an output if the name
# is misspelled.

try:
    x = numpy.array([[1.0, 2.0], [3.0, 4.0], [5.0, 6.0]], dtype=numpy.float32)
    sess.run(["misspelled"], {input_name: x})
except Exception as e:
    print("Misspelled output name")
    print(f"{type(e)}: {e}")

###########################
# The output name is optional, it can be replaced by *None*
# and *onnxruntime* will then return all the outputs.

x = numpy.array([[1.0, 2.0], [3.0, 4.0], [5.0, 6.0]], dtype=numpy.float32)
try:
    res = sess.run(None, {input_name: x})
    print("All outputs")
    print(res)
except (RuntimeError, InvalidArgument) as e:
    print(e)

#########################
# The same goes if the input name is misspelled.

try:
    x = numpy.array([[1.0, 2.0], [3.0, 4.0], [5.0, 6.0]], dtype=numpy.float32)
    sess.run([output_name], {"misspelled": x})
except Exception as e:
    print("Misspelled input name")
    print(f"{type(e)}: {e}")

#########################
# *onnxruntime* does not necessarily fail if the input
# dimension is a multiple of the expected input dimension.

for x in [
    numpy.array([1.0, 2.0, 3.0, 4.0], dtype=numpy.float32),
    numpy.array([[1.0, 2.0, 3.0, 4.0]], dtype=numpy.float32),
    numpy.array([[1.0, 2.0], [3.0, 4.0]], dtype=numpy.float32),
    numpy.array([1.0, 2.0, 3.0], dtype=numpy.float32),
    numpy.array([[1.0, 2.0, 3.0]], dtype=numpy.float32),
]:
    try:
        r = sess.run([output_name], {input_name: x})
        print(f"Shape={x.shape} and predicted labels={r}")
    except (RuntimeError, InvalidArgument) as e:
        print(f"ERROR with Shape={x.shape} - {e}")

for x in [
    numpy.array([1.0, 2.0, 3.0, 4.0], dtype=numpy.float32),
    numpy.array([[1.0, 2.0, 3.0, 4.0]], dtype=numpy.float32),
    numpy.array([[1.0, 2.0], [3.0, 4.0]], dtype=numpy.float32),
    numpy.array([1.0, 2.0, 3.0], dtype=numpy.float32),
    numpy.array([[1.0, 2.0, 3.0]], dtype=numpy.float32),
]:
    try:
        r = sess.run(None, {input_name: x})
        print(f"Shape={x.shape} and predicted probabilities={r[1]}")
    except (RuntimeError, InvalidArgument) as e:
        print(f"ERROR with Shape={x.shape} - {e}")

#########################
# It does not fail either if the number of dimension
# is higher than expects but produces a warning.

for x in [
    numpy.array([[[1.0, 2.0], [3.0, 4.0]]], dtype=numpy.float32),
    numpy.array([[[1.0, 2.0, 3.0]]], dtype=numpy.float32),
    numpy.array([[[1.0, 2.0]], [[3.0, 4.0]]], dtype=numpy.float32),
]:
    try:
        r = sess.run([output_name], {input_name: x})
        print(f"Shape={x.shape} and predicted labels={r}")
    except (RuntimeError, InvalidArgument) as e:
        print(f"ERROR with Shape={x.shape} - {e}")
